@@ -1,6 +1,6 @@
 pycbc_inference \
---config-file `dirname "$0"`/single.ini \
---nprocesses=1 \
+--config-file `dirname "$0"`/single_simple.ini \
+--nprocesses=2 \
 --output-file single.hdf \
 --seed 0 \
 --force \
