@@ -1,6 +1,7 @@
+import matplotlib.pyplot as pp
 import pycbc.noise
 import pycbc.psd
-import pylab
+
 
 # The color of the noise matches a PSD which you provide
 flow = 30.0
@@ -13,7 +14,7 @@ delta_t = 1.0 / 4096
 tsamples = int(32 / delta_t)
 ts = pycbc.noise.noise_from_psd(tsamples, delta_t, psd, seed=127)
 
-pylab.plot(ts.sample_times, ts)
-pylab.ylabel('Strain')
-pylab.xlabel('Time (s)')
-pylab.show()
+pp.plot(ts.sample_times, ts)
+pp.ylabel('Strain')
+pp.xlabel('Time (s)')
+pp.show()
