@@ -1,3 +1,3 @@
 pycbc_submit_dax --no-grid --no-create-proxy \
 --local-dir ./ \
---no-query-db
+--no-query-db \
