@@ -27,8 +27,11 @@ values.
 """
 import logging
 import numpy
+import h5py
 from . import ranking
 from . import coinc_rate
+from .eventmgr_cython import logsignalrateinternals_computepsignalbins
+from .eventmgr_cython import logsignalrateinternals_compute2detrate
 
 
 class Stat(object):
@@ -50,20 +53,19 @@ class Stat(object):
         ifos: list of strs, needed for some statistics
             The list of detector names
         """
-        import h5py
 
         self.files = {}
         files = files or []
         for filename in files:
-            f = h5py.File(filename, 'r')
-            stat = f.attrs['stat']
+            with h5py.File(filename, 'r') as f:
+                stat = f.attrs['stat']
             if hasattr(stat, 'decode'):
                 stat = stat.decode()
             if stat in self.files:
                 raise RuntimeError("We already have one file with stat attr ="
                                    " %s. Can't provide more than one!" % stat)
             logging.info("Found file %s for stat %s", filename, stat)
-            self.files[stat] = f
+            self.files[stat] = filename
 
         # Provide the dtype of the single detector method's output
         # This is used by background estimation codes that need to maintain
@@ -87,8 +89,8 @@ class Stat(object):
 
         Parameters
         ----------
-        trigs: dict of numpy.ndarrays, h5py group (or similar dict-like object)
-            Dictionary-like object holding single detector trigger information.
+        trigs: dict of numpy.ndarrays, h5py group or similar dict-like object
+            Object holding single detector trigger information.
 
         Returns
         -------
@@ -107,8 +109,8 @@ class Stat(object):
 
         Parameters
         ----------
-        trigs: dict of numpy.ndarrays, h5py group (or similar dict-like object)
-            Dictionary-like object holding single detector trigger information.
+        trigs: dict of numpy.ndarrays, h5py group or similar dict-like object
+            Object holding single detector trigger information.
 
         Returns
         -------
@@ -193,8 +195,8 @@ class QuadratureSumStatistic(Stat):
 
         Parameters
         ----------
-        trigs: dict of numpy.ndarrays, h5py group (or similar dict-like object)
-            Dictionary-like object holding single detector trigger information.
+        trigs: dict of numpy.ndarrays, h5py group or similar dict-like object
+            Object holding single detector trigger information.
 
         Returns
         -------
@@ -290,8 +292,6 @@ class PhaseTDStatistic(QuadratureSumStatistic):
     def __init__(self, sngl_ranking, files=None, ifos=None,
                  pregenerate_hist=True, **kwargs):
         """
-        Create a statistic class instance
-
         Parameters
         ----------
         sngl_ranking: str
@@ -335,6 +335,14 @@ class PhaseTDStatistic(QuadratureSumStatistic):
         self.param_bin = {}
         self.two_det_flag = (len(ifos) == 2)
         self.two_det_weights = {}
+        # Some memory
+        self.pdif = numpy.zeros(128, dtype=numpy.float64)
+        self.tdif = numpy.zeros(128, dtype=numpy.float64)
+        self.sdif = numpy.zeros(128, dtype=numpy.float64)
+        self.tbin = numpy.zeros(128, dtype=numpy.int32)
+        self.pbin = numpy.zeros(128, dtype=numpy.int32)
+        self.sbin = numpy.zeros(128, dtype=numpy.int32)
+
         if pregenerate_hist and not len(ifos) == 1:
             self.get_hist()
 
@@ -371,22 +379,32 @@ class PhaseTDStatistic(QuadratureSumStatistic):
             raise RuntimeError("Couldn't figure out which stat file to use")
 
         logging.info("Using signal histogram %s for ifos %s", selected, ifos)
-        histfile = self.files[selected]
-        self.hist_ifos = histfile.attrs['ifos']
-        # Patch for pre-hdf5=3.0 histogram files
-        try:
-            logging.info("Decoding hist ifos ..")
-            self.hist_ifos = [i.decode('UTF-8') for i in self.hist_ifos]
-        except (UnicodeDecodeError, AttributeError):
-            pass
-        n_ifos = len(self.hist_ifos)
+        weights = {}
+        param = {}
 
-        # Histogram bin attributes
-        self.twidth = histfile.attrs['twidth']
-        self.pwidth = histfile.attrs['pwidth']
-        self.swidth = histfile.attrs['swidth']
-        self.srbmin = histfile.attrs['srbmin']
-        self.srbmax = histfile.attrs['srbmax']
+        with h5py.File(self.files[selected], 'r') as histfile:
+            self.hist_ifos = histfile.attrs['ifos']
+
+            # Patch for pre-hdf5=3.0 histogram files
+            try:
+                logging.info("Decoding hist ifos ..")
+                self.hist_ifos = [i.decode('UTF-8') for i in self.hist_ifos]
+            except (UnicodeDecodeError, AttributeError):
+                pass
+
+            # Histogram bin attributes
+            self.twidth = histfile.attrs['twidth']
+            self.pwidth = histfile.attrs['pwidth']
+            self.swidth = histfile.attrs['swidth']
+            self.srbmin = histfile.attrs['srbmin']
+            self.srbmax = histfile.attrs['srbmax']
+            relfac = histfile.attrs['sensitivity_ratios']
+
+            for ifo in self.hist_ifos:
+                weights[ifo] = histfile[ifo]['weights'][:]
+                param[ifo] = histfile[ifo]['param_bin'][:]
+
+        n_ifos = len(self.hist_ifos)
 
         bin_volume = (self.twidth * self.pwidth * self.swidth) ** (n_ifos - 1)
         self.hist_max = - 1. * numpy.inf
@@ -395,20 +413,18 @@ class PhaseTDStatistic(QuadratureSumStatistic):
         # the coinc
         for ifo in self.hist_ifos:
 
-            weights = histfile[ifo]['weights'][:]
             # renormalise to PDF
-            self.weights[ifo] = weights / (weights.sum() * bin_volume)
+            self.weights[ifo] = \
+                weights[ifo] / (weights[ifo].sum() * bin_volume)
 
-            param = histfile[ifo]['param_bin'][:]
-
-            if param.dtype == numpy.int8:
+            if param[ifo].dtype == numpy.int8:
                 # Older style, incorrectly sorted histogram file
-                ncol = param.shape[1]
-                self.pdtype = [('c%s' % i, param.dtype) for i in range(ncol)]
+                ncol = param[ifo].shape[1]
+                self.pdtype = [('c%s' % i, param[ifo].dtype) for i in range(ncol)]
                 self.param_bin[ifo] = numpy.zeros(len(self.weights[ifo]),
                                                   dtype=self.pdtype)
                 for i in range(ncol):
-                    self.param_bin[ifo]['c%s' % i] = param[:, i]
+                    self.param_bin[ifo]['c%s' % i] = param[ifo][:, i]
 
                 lsort = self.param_bin[ifo].argsort()
                 self.param_bin[ifo] = self.param_bin[ifo][lsort]
@@ -416,7 +432,7 @@ class PhaseTDStatistic(QuadratureSumStatistic):
             else:
                 # New style, efficient histogram file
                 # param bin and weights have already been sorted
-                self.param_bin[ifo] = param
+                self.param_bin[ifo] = param[ifo]
                 self.pdtype = self.param_bin[ifo].dtype
 
             # Max_penalty is a small number to assigned to any bins without
@@ -446,12 +462,15 @@ class PhaseTDStatistic(QuadratureSumStatistic):
                     self.c1_size = {}
                     self.c2_size = {}
 
-                self.c0_size[ifo] = 2 * (abs(self.param_bin[ifo]['c0']).max()
-                                         + 1)
-                self.c1_size[ifo] = 2 * (abs(self.param_bin[ifo]['c1']).max()
-                                         + 1)
-                self.c2_size[ifo] = 2 * (abs(self.param_bin[ifo]['c2']).max()
-                                         + 1)
+                self.c0_size[ifo] = numpy.int32(
+                    2 * (abs(self.param_bin[ifo]['c0']).max() + 1)
+                )
+                self.c1_size[ifo] = numpy.int32(
+                    2 * (abs(self.param_bin[ifo]['c1']).max() + 1)
+                )
+                self.c2_size[ifo] = numpy.int32(
+                    2 * (abs(self.param_bin[ifo]['c2']).max() + 1)
+                )
 
                 array_size = [self.c0_size[ifo], self.c1_size[ifo],
                               self.c2_size[ifo]]
@@ -466,7 +485,6 @@ class PhaseTDStatistic(QuadratureSumStatistic):
                     + self.c2_size[ifo] // 2
                 self.two_det_weights[ifo][id0, id1, id2] = self.weights[ifo]
 
-        relfac = histfile.attrs['sensitivity_ratios']
         for ifo, sense in zip(self.hist_ifos, relfac):
             self.relsense[ifo] = sense
 
@@ -501,74 +519,103 @@ class PhaseTDStatistic(QuadratureSumStatistic):
         # to use as reference for choosing the signal histogram.
         snrs = numpy.array([numpy.array(stats[ifo]['snr'], ndmin=1)
                            for ifo in self.ifos])
-        smin = numpy.argmin(snrs, axis=0)
+        smin = snrs.argmin(axis=0)
         # Store a list of the triggers using each ifo as reference
         rtypes = {ifo: numpy.where(smin == j)[0]
                   for j, ifo in enumerate(self.ifos)}
 
         # Get reference ifo information
         rate = numpy.zeros(len(shift), dtype=numpy.float32)
+        ps = {ifo: numpy.array(stats[ifo]['coa_phase'], ndmin=1)
+              for ifo in self.ifos}
+        ts = {ifo: numpy.array(stats[ifo]['end_time'], ndmin=1)
+              for ifo in self.ifos}
+        ss = {ifo: numpy.array(stats[ifo]['snr'], ndmin=1)
+              for ifo in self.ifos}
+        sigs = {ifo: numpy.array(stats[ifo]['sigmasq'], ndmin=1)
+                for ifo in self.ifos}
         for ref_ifo in self.ifos:
             rtype = rtypes[ref_ifo]
-            ref = stats[ref_ifo]
-            pref = numpy.array(ref['coa_phase'], ndmin=1)[rtype]
-            tref = numpy.array(ref['end_time'], ndmin=1)[rtype]
-            sref = numpy.array(ref['snr'], ndmin=1)[rtype]
-            sigref = numpy.array(ref['sigmasq'], ndmin=1) ** 0.5
-            sigref = sigref[rtype]
+            pref = ps[ref_ifo]
+            tref = ts[ref_ifo]
+            sref = ss[ref_ifo]
+            sigref = sigs[ref_ifo]
             senseref = self.relsense[self.hist_ifos[0]]
 
             binned = []
             other_ifos = [ifo for ifo in self.ifos if ifo != ref_ifo]
             for ifo in other_ifos:
-                sc = stats[ifo]
-                p = numpy.array(sc['coa_phase'], ndmin=1)[rtype]
-                t = numpy.array(sc['end_time'], ndmin=1)[rtype]
-                s = numpy.array(sc['snr'], ndmin=1)[rtype]
-
-                sense = self.relsense[ifo]
-                sig = numpy.array(sc['sigmasq'], ndmin=1) ** 0.5
-                sig = sig[rtype]
+                # Assign cached memory
+                length = len(rtype)
+                while length > len(self.pdif):
+                    newlen = len(self.pdif) * 2
+                    self.pdif = numpy.zeros(newlen, dtype=numpy.float64)
+                    self.tdif = numpy.zeros(newlen, dtype=numpy.float64)
+                    self.sdif = numpy.zeros(newlen, dtype=numpy.float64)
+                    self.pbin = numpy.zeros(newlen, dtype=numpy.int32)
+                    self.tbin = numpy.zeros(newlen, dtype=numpy.int32)
+                    self.sbin = numpy.zeros(newlen, dtype=numpy.int32)
 
                 # Calculate differences
-                pdif = (pref - p) % (numpy.pi * 2.0)
-                tdif = shift[rtype] * to_shift[ref_ifo] + \
-                    tref - shift[rtype] * to_shift[ifo] - t
-                sdif = s / sref * sense / senseref * sigref / sig
+                logsignalrateinternals_computepsignalbins(
+                    self.pdif,
+                    self.tdif,
+                    self.sdif,
+                    self.pbin,
+                    self.tbin,
+                    self.sbin,
+                    ps[ifo],
+                    ts[ifo],
+                    ss[ifo],
+                    sigs[ifo],
+                    pref,
+                    tref,
+                    sref,
+                    sigref,
+                    shift,
+                    rtype,
+                    self.relsense[ifo],
+                    senseref,
+                    self.twidth,
+                    self.pwidth,
+                    self.swidth,
+                    to_shift[ref_ifo],
+                    to_shift[ifo],
+                    length
+                )
 
-                # Put into bins
-                tbin = (tdif / self.twidth).astype(int)
-                pbin = (pdif / self.pwidth).astype(int)
-                sbin = (sdif / self.swidth).astype(int)
-                binned += [tbin, pbin, sbin]
-
-            # Convert binned to same dtype as stored in hist
-            nbinned = numpy.zeros(len(pbin), dtype=self.pdtype)
-            for i, b in enumerate(binned):
-                nbinned['c%s' % i] = b
+                binned += [
+                    self.tbin[:length],
+                    self.pbin[:length],
+                    self.sbin[:length]
+                ]
 
             # Read signal weight from precalculated histogram
             if self.two_det_flag:
                 # High-RAM, low-CPU option for two-det
-                rate[rtype] = numpy.zeros(len(nbinned)) + self.max_penalty
-
-                id0 = nbinned['c0'].astype(numpy.int32) \
-                    + self.c0_size[ref_ifo] // 2
-                id1 = nbinned['c1'].astype(numpy.int32) \
-                    + self.c1_size[ref_ifo] // 2
-                id2 = nbinned['c2'].astype(numpy.int32) \
-                    + self.c2_size[ref_ifo] // 2
-
-                # look up keys which are within boundaries
-                within = (id0 > 0) & (id0 < self.c0_size[ref_ifo])
-                within = within & (id1 > 0) & (id1 < self.c1_size[ref_ifo])
-                within = within & (id2 > 0) & (id2 < self.c2_size[ref_ifo])
-                within = numpy.where(within)[0]
-                rate[rtype[within]] = \
-                    self.two_det_weights[ref_ifo][id0[within], id1[within],
-                                                  id2[within]]
+                logsignalrateinternals_compute2detrate(
+                    binned[0],
+                    binned[1],
+                    binned[2],
+                    self.c0_size[ref_ifo],
+                    self.c1_size[ref_ifo],
+                    self.c2_size[ref_ifo],
+                    rate,
+                    rtype,
+                    sref,
+                    self.two_det_weights[ref_ifo],
+                    self.max_penalty,
+                    self.ref_snr,
+                    len(rtype)
+                )
             else:
                 # Low[er]-RAM, high[er]-CPU option for >two det
+
+                # Convert binned to same dtype as stored in hist
+                nbinned = numpy.zeros(len(binned[1]), dtype=self.pdtype)
+                for i, b in enumerate(binned):
+                    nbinned[f'c{i}'] = b
+
                 loc = numpy.searchsorted(self.param_bin[ref_ifo], nbinned)
                 loc[loc == len(self.weights[ref_ifo])] = 0
                 rate[rtype] = self.weights[ref_ifo][loc]
@@ -579,9 +626,8 @@ class PhaseTDStatistic(QuadratureSumStatistic):
                     self.param_bin[ref_ifo][loc] != nbinned
                 )[0]
                 rate[rtype[missed]] = self.max_penalty
-
-            # Scale by signal population SNR
-            rate[rtype] *= (sref / self.ref_snr) ** -4.0
+                # Scale by signal population SNR
+                rate[rtype] *= (sref[rtype] / self.ref_snr) ** -4.0
 
         return numpy.log(rate)
 
@@ -674,8 +720,6 @@ class ExpFitStatistic(QuadratureSumStatistic):
 
     def __init__(self, sngl_ranking, files=None, ifos=None, **kwargs):
         """
-        Create a statistic class instance
-
         Parameters
         ----------
         sngl_ranking: str
@@ -726,7 +770,7 @@ class ExpFitStatistic(QuadratureSumStatistic):
             A dictionary containing the fit information in the `alpha`, `rate`
             and `thresh` keys/.
         """
-        coeff_file = self.files[ifo+'-fit_coeffs']
+        coeff_file = h5py.File(self.files[f'{ifo}-fit_coeffs'], 'r')
         template_id = coeff_file['template_id'][:]
         # the template_ids and fit coeffs are stored in an arbitrary order
         # create new arrays in template_id order for easier recall
@@ -753,6 +797,8 @@ class ExpFitStatistic(QuadratureSumStatistic):
         # Keep the fit threshold in fits_by_tid
         fits_by_tid_dict['thresh'] = coeff_file.attrs['stat_threshold']
 
+        coeff_file.close()
+
         return fits_by_tid_dict
 
     def get_ref_vals(self, ifo):
@@ -774,8 +820,8 @@ class ExpFitStatistic(QuadratureSumStatistic):
 
         Parameters
         ----------
-        trigs: dict of numpy.ndarrays, h5py group (or similar dict-like object)
-            Dictionary-like object holding single detector trigger information.
+        trigs: dict of numpy.ndarrays, h5py group or similar dict-like object
+            Object holding single detector trigger information.
             The coincidence executable will always call this using a bunch of
             trigs from a single template, there template_num is stored as an
             attribute and we just return the single value for all templates.
@@ -816,8 +862,8 @@ class ExpFitStatistic(QuadratureSumStatistic):
 
         Parameters
         -----------
-        trigs: dict of numpy.ndarrays, h5py group (or similar dict-like object)
-            Dictionary-like object holding single detector trigger information.
+        trigs: dict of numpy.ndarrays, h5py group or similar dict-like object
+            Object holding single detector trigger information.
 
         Returns
         ---------
@@ -843,8 +889,8 @@ class ExpFitStatistic(QuadratureSumStatistic):
 
         Parameters
         ----------
-        trigs: dict of numpy.ndarrays, h5py group (or similar dict-like object)
-            Dictionary-like object holding single detector trigger information.
+        trigs: dict of numpy.ndarrays, h5py group or similar dict-like object
+            Object holding single detector trigger information.
 
         Returns
         -------
@@ -938,8 +984,6 @@ class ExpFitCombinedSNR(ExpFitStatistic):
 
     def __init__(self, sngl_ranking, files=None, ifos=None, **kwargs):
         """
-        Create a statistic class instance
-
         Parameters
         ----------
         sngl_ranking: str
@@ -978,8 +1022,8 @@ class ExpFitCombinedSNR(ExpFitStatistic):
 
         Parameters
         ----------
-        trigs: dict of numpy.ndarrays, h5py group (or similar dict-like object)
-            Dictionary-like object holding single detector trigger information.
+        trigs: dict of numpy.ndarrays, h5py group or similar dict-like object
+            Object holding single detector trigger information.
 
         Returns
         -------
@@ -1076,11 +1120,8 @@ class PhaseTDExpFitStatistic(PhaseTDStatistic, ExpFitCombinedSNR):
     Statistic combining exponential noise model with signal histogram PDF
     """
 
-    # default is 2-ifo operation with exactly 1 'phasetd' file
     def __init__(self, sngl_ranking, files=None, ifos=None, **kwargs):
         """
-        Create a statistic class instance
-
         Parameters
         ----------
         sngl_ranking: str
@@ -1110,8 +1151,8 @@ class PhaseTDExpFitStatistic(PhaseTDStatistic, ExpFitCombinedSNR):
 
         Parameters
         ----------
-        trigs: dict of numpy.ndarrays, h5py group (or similar dict-like object)
-            Dictionary-like object holding single detector trigger information.
+        trigs: dict of numpy.ndarrays, h5py group or similar dict-like object
+            Object holding single detector trigger information.
 
         Returns
         -------
@@ -1200,8 +1241,6 @@ class ExpFitBgRateStatistic(ExpFitStatistic):
     def __init__(self, sngl_ranking, files=None, ifos=None,
                  benchmark_lograte=-14.6, **kwargs):
         """
-        Create a statistic class instance
-
         Parameters
         ----------
         sngl_ranking: str
@@ -1240,12 +1279,14 @@ class ExpFitBgRateStatistic(ExpFitStatistic):
         ifo: str
             The ifo to consider.
         """
-        coeff_file = self.files[ifo+'-fit_coeffs']
-        analysis_time = float(coeff_file.attrs['analysis_time'])
+        with h5py.File(self.files[f'{ifo}-fit_coeffs'], 'r') as coeff_file:
+            analysis_time = float(coeff_file.attrs['analysis_time'])
+            fbt = 'fit_by_template' in coeff_file
+
         self.fits_by_tid[ifo]['smoothed_rate_above_thresh'] /= analysis_time
         self.fits_by_tid[ifo]['smoothed_rate_in_template'] /= analysis_time
         # The by-template fits may have been stored in the smoothed fits file
-        if 'fit_by_template' in coeff_file:
+        if fbt:
             self.fits_by_tid[ifo]['fit_by_rate_above_thresh'] /= analysis_time
             self.fits_by_tid[ifo]['fit_by_rate_in_template'] /= analysis_time
 
@@ -1323,8 +1364,6 @@ class ExpFitFgBgNormStatistic(PhaseTDStatistic,
     def __init__(self, sngl_ranking, files=None, ifos=None,
                  reference_ifos='H1,L1', **kwargs):
         """
-        Create a statistic class instance
-
         Parameters
         ----------
         sngl_ranking: str
@@ -1372,11 +1411,11 @@ class ExpFitFgBgNormStatistic(PhaseTDStatistic,
             The ifo to consider.
         """
 
-        coeff_file = self.files[ifo + '-fit_coeffs']
-        template_id = coeff_file['template_id'][:]
-        tid_sort = numpy.argsort(template_id)
-        self.fits_by_tid[ifo]['median_sigma'] = \
-            coeff_file['median_sigma'][:][tid_sort]
+        with h5py.File(self.files[f'{ifo}-fit_coeffs'], 'r') as coeff_file:
+            template_id = coeff_file['template_id'][:]
+            tid_sort = numpy.argsort(template_id)
+            self.fits_by_tid[ifo]['median_sigma'] = \
+                coeff_file['median_sigma'][:][tid_sort]
 
     def lognoiserate(self, trigs, alphabelow=6):
         """
@@ -1387,8 +1426,8 @@ class ExpFitFgBgNormStatistic(PhaseTDStatistic,
 
         Parameters
         -----------
-        trigs: dict of numpy.ndarrays, h5py group (or similar dict-like object)
-            Dictionary-like object holding single detector trigger information.
+        trigs: dict of numpy.ndarrays, h5py group or similar dict-like object
+            Object holding single detector trigger information.
         alphabelow: float, default=6
             Use this slope to fit the noise triggers below the point at which
             fits are present in the input files.
@@ -1420,8 +1459,8 @@ class ExpFitFgBgNormStatistic(PhaseTDStatistic,
 
         Parameters
         ----------
-        trigs: dict of numpy.ndarrays, h5py group (or similar dict-like object)
-            Dictionary-like object holding single detector trigger information.
+        trigs: dict of numpy.ndarrays, h5py group or similar dict-like object
+            Object holding single detector trigger information.
         Returns
         -------
         numpy.ndarray
@@ -1576,7 +1615,8 @@ class ExpFitFgBgNormStatistic(PhaseTDStatistic,
         # Safety against subclassing and not rethinking this
         allowed_names = ['ExpFitFgBgNormStatistic',
                          'ExpFitFgBgNormBBHStatistic',
-                         'DQExpFitFgBgNormStatistic']
+                         'DQExpFitFgBgNormStatistic',
+                         'ExpFitFgBgKDEStatistic']
         self._check_coinc_lim_subclass(allowed_names)
 
         if not self.has_hist:
@@ -1642,8 +1682,6 @@ class ExpFitFgBgNormBBHStatistic(ExpFitFgBgNormStatistic):
     def __init__(self, sngl_ranking, files=None, ifos=None,
                  max_chirp_mass=None, **kwargs):
         """
-        Create a statistic class instance
-
         Parameters
         ----------
         sngl_ranking: str
@@ -1688,8 +1726,7 @@ class ExpFitFgBgNormBBHStatistic(ExpFitFgBgNormStatistic):
         value: log of coinc signal rate density for the given single-ifo
             triggers and time shifts
         """
-
-        # model signal rate as uniform over chirp mass, background rate is
+        # Model signal rate as uniform over chirp mass, background rate is
         # proportional to mchirp^(-11/3) due to density of templates
         logr_s = ExpFitFgBgNormStatistic.logsignalrate(
                     self,
@@ -1711,8 +1748,8 @@ class ExpFitFgBgNormBBHStatistic(ExpFitFgBgNormStatistic):
 
         Parameters
         ----------
-        trigs: dict of numpy.ndarrays, h5py group (or similar dict-like object)
-            Dictionary-like object holding single detector trigger information.
+        trigs: dict of numpy.ndarrays, h5py group or similar dict-like object
+            Object holding single detector trigger information.
 
         Returns
         -------
@@ -1758,20 +1795,154 @@ class ExpFitFgBgNormBBHStatistic(ExpFitFgBgNormStatistic):
         return loglr
 
 
+class ExpFitFgBgKDEStatistic(ExpFitFgBgNormStatistic):
+    """
+    The ExpFitFgBgNormStatistic with an additional mass and spin weighting
+    factor determined by KDE statistic files.
+
+    This is the same as the ExpFitFgBgNormStatistic except the likelihood
+    ratio is multiplied by the ratio of signal KDE to template KDE over some
+    parameters covering the bank.
+    """
+
+    def __init__(self, sngl_ranking, files=None, ifos=None, **kwargs):
+        """
+        Parameters
+        ----------
+        sngl_ranking: str
+            The name of the ranking to use for the single-detector triggers.
+        files: list of strs, needed here
+            A list containing the filenames of hdf format files used to help
+            construct the coincident statistics. The files must have a 'stat'
+            attribute which is used to associate them with the appropriate
+            statistic class.
+        ifos: list of strs, not used here
+            The list of detector names
+        """
+        ExpFitFgBgNormStatistic.__init__(self, sngl_ranking, files=files,
+                                         ifos=ifos, **kwargs)
+        # The stat file attributes are hard-coded as 'signal-kde_file'
+        # and 'template-kde_file'
+        parsed_attrs = [f.split('-') for f in self.files.keys()]
+        self.kde_names = [at[0] for at in parsed_attrs if
+                       (len(at) == 2 and at[1] == 'kde_file')]
+        assert sorted(self.kde_names) == ['signal', 'template'], \
+            "Two stat files are required, they should have stat attr " \
+            "'signal-kde_file' and 'template-kde_file' respectively"
+
+        self.kde_by_tid = {}
+        for kname in self.kde_names:
+            self.assign_kdes(kname)
+        # This will hold the template ids of the events for the statistic
+        # calculation
+        self.curr_tnum = None
+
+    def assign_kdes(self, kname):
+        """
+        Extract values from KDE files
+
+        Parameters
+        -----------
+        kname: str
+            Used to label the kde files.
+        """
+        with h5py.File(self.files[kname+'-kde_file'], 'r') as kde_file:
+            self.kde_by_tid[kname+'_kdevals'] = kde_file['data_kde'][:]
+
+    def single(self, trigs):
+        """
+        Calculate the necessary single detector information including getting
+        template ids from single detector triggers.
+
+        Parameters
+        ----------
+        trigs: dict of numpy.ndarrays, h5py group or similar dict-like object
+            Object holding single detector trigger information
+
+        Returns
+        -------
+        numpy.ndarray
+            The array of single detector values
+        """
+        try:
+            # template_num exists if accessed via coinc_findtrigs
+            self.curr_tnum = trigs.template_num
+        except AttributeError:
+            # exists for SingleDetTriggers
+            self.curr_tnum = trigs['template_id']
+        return ExpFitFgBgNormStatistic.single(self, trigs)
+
+    def logsignalrate(self, stats, shift, to_shift):
+        """
+        Calculate the normalized log rate density of signals via lookup.
+
+        This calls back to the parent class and then applies the ratio_kde
+        weighting factor.
+
+        Parameters
+        ----------
+        stats: list of dicts giving single-ifo quantities, ordered as
+            self.ifos
+        shift: numpy array of float, size of the time shift vector for each
+            coinc to be ranked
+        to_shift: list of int, multiple of the time shift to apply ordered
+            as self.ifos
+
+        Returns
+        -------
+        value: log of coinc signal rate density for the given single-ifo
+            triggers and time shifts
+        """
+        logr_s = ExpFitFgBgNormStatistic.logsignalrate(self, stats, shift,
+                                                       to_shift)
+        signal_kde = self.kde_by_tid["signal_kdevals"][self.curr_tnum]
+        template_kde = self.kde_by_tid["template_kdevals"][self.curr_tnum]
+        logr_s += numpy.log(signal_kde / template_kde)
+        return logr_s
+
+    def coinc_lim_for_thresh(self, s, thresh, limifo, **kwargs):
+        """
+        Optimization function to identify coincs too quiet to be of interest
+
+        Calculate the required single detector statistic to exceed the
+        threshold for each of the input trigers.
+
+        Parameters
+        ----------
+        s: list
+            List of (ifo, single detector statistic) tuples for all detectors
+            except limifo.
+        thresh: float
+            The threshold on the coincident statistic.
+        limifo: string
+            The ifo for which the limit is to be found.
+
+        Returns
+        -------
+        numpy.ndarray
+            Array of limits on the limifo single statistic to
+            exceed thresh.
+        """
+        loglr = ExpFitFgBgNormStatistic.coinc_lim_for_thresh(
+            self, s, thresh, limifo, **kwargs)
+        signal_kde = self.kde_by_tid["signal_kdevals"][self.curr_tnum]
+        template_kde = self.kde_by_tid["template_kdevals"][self.curr_tnum]
+        loglr += numpy.log(signal_kde / template_kde)
+        return loglr
+
+
 class DQExpFitFgBgNormStatistic(ExpFitFgBgNormStatistic):
     """
     The ExpFitFgBgNormStatistic with DQ-based reranking.
 
     This is the same as the ExpFitFgBgNormStatistic except the likelihood
-    is multiplied by the relative signal rate based on the relevant
-    DQ likelihood value.
+    ratio is corrected via estimating relative noise trigger rates based on
+    the DQ time series.
     """
 
     def __init__(self, sngl_ranking, files=None, ifos=None,
                  **kwargs):
         """
-        Create a statistic class instance
-
         Parameters
         ----------
         sngl_ranking: str
@@ -1820,14 +1991,15 @@ class DQExpFitFgBgNormStatistic(ExpFitFgBgNormStatistic):
             Dictionary containing the bin name for each template id
         """
         ifo = key.split('-')[0]
-        dq_file = self.files[key]
-        bin_names = dq_file.attrs['names'][:]
-        locs = []
-        names = []
-        for bin_name in bin_names:
-            bin_locs = dq_file[ifo + '/locs/' + bin_name][:]
-            locs = list(locs)+list(bin_locs.astype(int))
-            names = list(names)+list([bin_name]*len(bin_locs))
+        with h5py.File(self.files[key], 'r') as dq_file:
+            bin_names = dq_file.attrs['names'][:]
+            locs = []
+            names = []
+            for bin_name in bin_names:
+                bin_locs = dq_file[ifo + '/locs/' + bin_name][:]
+                locs = list(locs)+list(bin_locs.astype(int))
+                names = list(names)+list([bin_name]*len(bin_locs))
+
         bin_dict = dict(zip(locs, names))
         return bin_dict
 
@@ -1849,13 +2021,14 @@ class DQExpFitFgBgNormStatistic(ExpFitFgBgNormStatistic):
 
         """
         ifo = key.split('-')[0]
-        dq_file = self.files[key]
-        times = dq_file[ifo+'/times'][:]
-        bin_names = dq_file.attrs['names'][:]
-        dq_dict = {}
-        for bin_name in bin_names:
-            dq_vals = dq_file[ifo+'/dq_vals/'+bin_name][:]
-            dq_dict[bin_name] = dict(zip(times, dq_vals))
+        with h5py.File(self.files[key], 'r') as dq_file:
+            times = dq_file[ifo+'/times'][:]
+            bin_names = dq_file.attrs['names'][:]
+            dq_dict = {}
+            for bin_name in bin_names:
+                dq_vals = dq_file[ifo+'/dq_vals/'+bin_name][:]
+                dq_dict[bin_name] = dict(zip(times, dq_vals))
+
         return dq_dict
 
     def find_dq_val(self, trigs):
@@ -1890,8 +2063,8 @@ class DQExpFitFgBgNormStatistic(ExpFitFgBgNormStatistic):
 
         Parameters
         -----------
-        trigs: dict of numpy.ndarrays, h5py group (or similar dict-like object)
-            Dictionary-like object holding single detector trigger information.
+        trigs: dict of numpy.ndarrays, h5py group or similar dict-like object
+            Object holding single detector trigger information.
 
         Returns
         ---------
@@ -1915,6 +2088,7 @@ statistic_dict = {
     'exp_fit_bg_rate': ExpFitBgRateStatistic,
     'phasetd_exp_fit_fgbg_norm': ExpFitFgBgNormStatistic,
     'phasetd_exp_fit_fgbg_bbh_norm': ExpFitFgBgNormBBHStatistic,
+    'phasetd_exp_fit_fgbg_kde': ExpFitFgBgKDEStatistic,
 }
 
 
