@@ -28,6 +28,9 @@ exponential_fit = expfit(xvals, alpha, thresh)
 # or access function by name
 exponential_fit_1 = fit_fn('exponential', xvals, alpha, thresh)
 
+# Use weighting factors to e.g. take decimation into account
+alpha, sigma_alpha = fit_above_thresh('exponential', snrs, weights=weights)
+
 # get the KS test statistic and p-value - see scipy.stats.kstest
 ks_stat, ks_pval = KS_test('exponential', snrs, alpha, thresh)
 
@@ -45,24 +48,48 @@ ks_stat, ks_pval = KS_test('exponential', snrs, alpha, thresh)
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 # Public License for more details.
 
-from __future__ import division
+import logging
 import numpy
 from scipy.stats import kstest
 
+def exponential_fitalpha(vals, thresh, w):
+    """
+    Maximum likelihood estimator for the fit factor for
+    an exponential decrease model
+    """
+    return 1. / (numpy.average(vals, weights=w) - thresh)
+
+
+def rayleigh_fitalpha(vals, thresh, w):
+    """
+    Maximum likelihood estimator for the fit factor for
+    a Rayleigh distribution of events
+    """
+    return 2. / (numpy.average(vals ** 2., weights=w) - thresh ** 2.)
+
+
+def power_fitalpha(vals, thresh, w):
+    """
+    Maximum likelihood estimator for the fit factor for
+    a power law model
+    """
+    return numpy.average(numpy.log(vals/thresh), weights=w) ** -1. + 1.
+
+
 fitalpha_dict = {
-    'exponential' : lambda vals, thresh : 1. / (numpy.mean(vals) - thresh),
-    'rayleigh'    : lambda vals, thresh : 2. / (numpy.mean(vals**2.) - thresh**2.),
-    'power'       : lambda vals, thresh : numpy.mean(numpy.log(vals/thresh))**-1. + 1.
+    'exponential' : exponential_fitalpha,
+    'rayleigh'    : rayleigh_fitalpha,
+    'power'       : power_fitalpha
 }
 
 # measurement standard deviation = (-d^2 log L/d alpha^2)^(-1/2)
 fitstd_dict = {
-    'exponential' : lambda vals, alpha : alpha / len(vals)**0.5,
-    'rayleigh'    : lambda vals, alpha : alpha / len(vals)**0.5,
-    'power'       : lambda vals, alpha : (alpha - 1.) / len(vals)**0.5
+    'exponential' : lambda weights, alpha : alpha / sum(weights) ** 0.5,
+    'rayleigh'    : lambda weights, alpha : alpha / sum(weights) ** 0.5,
+    'power'       : lambda weights, alpha : (alpha - 1.) / sum(weights) ** 0.5
 }
 
-def fit_above_thresh(distr, vals, thresh=None):
+def fit_above_thresh(distr, vals, thresh=None, weights=None):
     """
     Maximum likelihood fit for the coefficient alpha
 
@@ -81,6 +108,9 @@ def fit_above_thresh(distr, vals, thresh=None):
         Values to fit
     thresh : float
         Threshold to apply before fitting; if None, use min(vals)
+    weights: sequence of floats
+        Weighting factors to use for the values when fitting.
+        Default=None - all the same
 
     Returns
     -------
@@ -92,17 +122,37 @@ def fit_above_thresh(distr, vals, thresh=None):
     vals = numpy.array(vals)
     if thresh is None:
         thresh = min(vals)
+        above_thresh = numpy.ones_like(vals, dtype=bool)
     else:
-        vals = vals[vals >= thresh]
-    alpha = fitalpha_dict[distr](vals, thresh)
-    return alpha, fitstd_dict[distr](vals, alpha)
+        above_thresh = vals >= thresh
+        if numpy.count_nonzero(above_thresh) == 0:
+            # Nothing is above threshold - warn and return -1
+            logging.warning("No values are above the threshold, %.2f, "
+                            "maximum is %.2f.", thresh, vals.max())
+            return -1., -1.
+
+        vals = vals[above_thresh]
+
+    # Set up the weights
+    if weights is not None:
+        weights = numpy.array(weights)
+        w = weights[above_thresh]
+    else:
+        w = numpy.ones_like(vals)
+
+    alpha = fitalpha_dict[distr](vals, thresh, w)
+    return alpha, fitstd_dict[distr](w, alpha)
 
 
+# Variables:
+# x: the trigger stat value(s) at which to evaluate the function
+# a: slope parameter of the fit
+# t: lower threshold stat value
 fitfn_dict = {
-    'exponential' : lambda x, alpha, t : alpha * numpy.exp(-alpha * (x - t)),
-    'rayleigh' : lambda x, alpha, t : (alpha * x * \
-                                       numpy.exp(-alpha * (x**2 - t**2) / 2.)),
-    'power' : lambda x, alpha, t : (alpha - 1.) * x**(-alpha) * t**(alpha - 1.)
+    'exponential' : lambda x, a, t : a * numpy.exp(-a * (x - t)),
+    'rayleigh' : lambda x, a, t : (a * x * \
+                                   numpy.exp(-a * (x ** 2 - t ** 2) / 2.)),
+    'power' : lambda x, a, t : (a - 1.) * x ** (-a) * t ** (a - 1.)
 }
 
 def fit_fn(distr, xvals, alpha, thresh):
@@ -134,8 +184,8 @@ def fit_fn(distr, xvals, alpha, thresh):
 
 cum_fndict = {
     'exponential' : lambda x, alpha, t : numpy.exp(-alpha * (x - t)),
-    'rayleigh' : lambda x, alpha, t : numpy.exp(-alpha * (x**2. - t**2.) / 2.),
-    'power' : lambda x, alpha, t : x**(1. - alpha) * t**(alpha - 1.)
+    'rayleigh' : lambda x, alpha, t : numpy.exp(-alpha * (x ** 2. - t ** 2.) / 2.),
+    'power' : lambda x, alpha, t : x ** (1. - alpha) * t ** (alpha - 1.)
 }
 
 def cum_fit(distr, xvals, alpha, thresh):
@@ -164,7 +214,6 @@ def cum_fit(distr, xvals, alpha, thresh):
     numpy.putmask(cum_fit, xvals < thresh, 0.)
     return cum_fit
 
-
 def tail_threshold(vals, N=1000):
     """Determine a threshold above which there are N louder values"""
     vals = numpy.array(vals)
