@@ -1,3 +1,4 @@
+import os
 from pycbc.types import zeros
 import numpy as _np
 import ctypes
@@ -12,6 +13,14 @@ from ..types import check_aligned
 # to set the threading backend, it is ESSENTIAL that simply loading this
 # module should not actually *call* ANY functions.
 
+# NOTE:
+# When loading FFTW we use os.RTLD_DEEPBIND to avoid potential segfaults due
+# to conflicts with MKL if both are present.
+if hasattr(os, 'RTLD_DEEPBIND'):
+    FFTW_RTLD_MODE = os.RTLD_DEEPBIND
+else:
+    FFTW_RTLD_MODE = ctypes.DEFAULT_MODE
+
 #FFTW constants, these are pulled from fftw3.h
 FFTW_FORWARD = -1
 FFTW_BACKWARD = 1
@@ -30,8 +39,8 @@ FFTW_WISDOM_ONLY = 1 << 21
 # We need to construct them directly with CDLL so
 # we can give the RTLD_GLOBAL mode, which we must do
 # in order to use the threaded libraries as well.
-double_lib = get_ctypes_library('fftw3',['fftw3'])
-float_lib = get_ctypes_library('fftw3f',['fftw3f'])
+double_lib = get_ctypes_library('fftw3', ['fftw3'], mode=FFTW_RTLD_MODE)
+float_lib = get_ctypes_library('fftw3f', ['fftw3f'], mode=FFTW_RTLD_MODE)
 if (double_lib is None) or (float_lib is None):
     raise ImportError("Unable to find FFTW libraries")
 
@@ -97,15 +106,24 @@ def _init_threads(backend):
         raise ValueError("Backend {0} for FFTW threading does not exist!".format(backend))
     if double_threaded_libname is not None:
         try:
-            # Note that the threaded libraries don't have their own pkg-config files;
-            # we must look for them wherever we look for double or single FFTW itself
-            _double_threaded_lib = get_ctypes_library(double_threaded_libname,
-                                                      ['fftw3'])
-            _float_threaded_lib =  get_ctypes_library(float_threaded_libname,
-                                                      ['fftw3f'])
+            # Note that the threaded libraries don't have their own pkg-config
+            # files we must look for them wherever we look for double or single
+            # FFTW itself.
+            _double_threaded_lib = get_ctypes_library(
+                double_threaded_libname,
+                ['fftw3'],
+                mode=FFTW_RTLD_MODE
+            )
+            _float_threaded_lib =  get_ctypes_library(
+                float_threaded_libname,
+                ['fftw3f'],
+                mode=FFTW_RTLD_MODE
+            )
             if (_double_threaded_lib is None) or (_float_threaded_lib is None):
-                raise RuntimeError("Unable to load threaded libraries {0} or {1}".format(double_threaded_libname,
-                                                                                         float_threaded_libname))
+                err_str = 'Unable to load threaded libraries'
+                err_str += f'{double_threaded_libname} or '
+                err_str += f'{float_threaded_libname}'
+                raise RuntimeError(err_str)
             dret = _double_threaded_lib.fftw_init_threads()
             fret = _float_threaded_lib.fftwf_init_threads()
             # FFTW for some reason uses *0* to indicate failure.  In C.
