@@ -208,7 +208,7 @@ def read_frame(location, channels, start_time=None,
     if check_integrity:
         stream.mode = (stream.mode | lalframe.FR_STREAM_CHECKSUM_MODE)
 
-    lalframe.FrSetMode(stream.mode, stream)
+    lalframe.FrStreamSetMode(stream, stream.mode)
 
     # determine duration of data
     if type(channels) is list:
@@ -681,6 +681,7 @@ class StatusBuffer(DataBuffer):
                             force_update_cache=force_update_cache,
                             increment_update_cache=increment_update_cache,
                             dtype=numpy.int32)
+
         self.valid_mask = valid_mask
         self.valid_on_zero = valid_on_zero
 
@@ -792,3 +793,116 @@ class StatusBuffer(DataBuffer):
         except RuntimeError:
             self.null_advance(blocksize)
             return False
+
+class iDQBuffer(object):
+
+    """ Read iDQ timeseries from a frame file """
+
+    def __init__(self, frame_src,
+                 idq_channel_name,
+                 idq_status_channel_name,
+                 idq_threshold,
+                 start_time,
+                 max_buffer=512,
+                 force_update_cache=False,
+                 increment_update_cache=None):
+        """
+        Parameters
+        ----------
+        frame_src: str of list of strings
+            Strings that indicate where to read from files from. This can be a
+            list of frame files, a glob, etc.
+        idq_channel_name: str
+            Name of the channel to read the iDQ statistic from
+        idq_status_channel_name: str
+            Name of the channel to read the iDQ status from
+        idq_threshold: float
+            Threshold which triggers a veto if iDQ channel falls below this threshold
+        start_time:
+            Time to start reading from.
+        max_buffer: {int, 512}, Optional
+            Length of the buffer in seconds
+        force_update_cache: {boolean, True}, Optional
+            Re-check the filesystem for frame files on every attempt to
+            read more data.
+        increment_update_cache: {str, None}, Optional
+            Pattern to look for frame files in a GPS dependent directory. This
+            is an alternate to the forced updated of the frame cache, and
+            apptempts to predict the next frame file name without probing the
+            filesystem.
+        """
+        self.threshold = idq_threshold
+        self.idq = DataBuffer(frame_src, idq_channel_name, start_time,
+                                max_buffer=max_buffer,
+                                force_update_cache=force_update_cache,
+                                increment_update_cache=increment_update_cache)
+        self.idq_state = DataBuffer(frame_src, idq_status_channel_name, start_time,
+                                        max_buffer=max_buffer,
+                                        force_update_cache=force_update_cache,
+                                        increment_update_cache=increment_update_cache)
+
+    def indices_of_flag(self, start_time, duration, times, padding=0):
+        """ Return the indices of the times lying in the flagged region
+
+        Parameters
+        ----------
+        start_time: int
+            Beginning time to request for
+        duration: int
+            Number of seconds to check.
+        padding: float
+            Number of seconds to add around flag inactive times to be considered
+        inactive as well.
+
+        Returns
+        -------
+        indices: numpy.ndarray
+            Array of indices marking the location of triggers within valid
+        time.
+        """
+        from pycbc.events.veto import indices_outside_times
+        sr = self.idq.raw_buffer.sample_rate
+        s = int((start_time - self.idq.raw_buffer.start_time - padding) * sr) - 1
+        e = s + int((duration + padding) * sr) + 1
+        idq_fap = self.idq.raw_buffer[s:e]
+        stamps = idq_fap.sample_times.numpy()
+        low_fap = idq_fap.numpy() <= self.threshold
+        idq_valid = self.idq_state.raw_buffer[s:e]
+        idq_valid = idq_valid.numpy().astype(bool)
+        valid_low_fap = numpy.logical_and(idq_valid, low_fap)
+        glitch_idx = numpy.flatnonzero(valid_low_fap)
+        glitch_times = stamps[glitch_idx]
+        starts = glitch_times - padding
+        ends = starts + 1.0 / sr + padding * 2.0
+        idx = indices_outside_times(times, starts, ends)
+        return idx
+
+    def advance(self, blocksize):
+        """ Add blocksize seconds more to the buffer, push blocksize seconds
+        from the beginning.
+
+        Parameters
+        ----------
+        blocksize: int
+            The number of seconds to attempt to read
+
+        Returns
+        -------
+        status: boolean
+            Returns True if advance is succesful,
+            False if not.
+        """
+        idq_ts = self.idq.attempt_advance(blocksize)
+        idq_state_ts = self.idq_state.attempt_advance(blocksize)
+        return (idq_ts is not None) and (idq_state_ts is not None)
+
+    def null_advance(self, blocksize):
+        """Advance and insert zeros
+
+        Parameters
+        ----------
+        blocksize: int
+            The number of seconds to advance the buffers
+        """
+        self.idq.null_advance(blocksize)
+        self.idq_state.null_advance(blocksize)
