@@ -15,23 +15,20 @@
 
 """Provides abstract base class for all samplers."""
 
-from __future__ import absolute_import
 
 import time
 from abc import (ABCMeta, abstractmethod)
 
-from six import add_metaclass
-
 from .base_hdf import BaseInferenceFile
 
 
-@add_metaclass(ABCMeta)
-class BaseSamplerFile(BaseInferenceFile):
+class BaseSamplerFile(BaseInferenceFile, metaclass=ABCMeta):
     """Base HDF class for all samplers.
 
     This adds abstract methods ``write_resume_point`` and
     ``write_sampler_metadata`` to :py:class:`BaseInferenceFile`.
     """
+
     def write_run_start_time(self):
         """Writes the current (UNIX) time to the file.
 
