@@ -345,7 +345,7 @@ class BaseModel(metaclass=ABCMeta):
     name = None
 
     def __init__(self, variable_params, static_params=None, prior=None,
-                 sampling_transforms=None, waveform_transforms=None):
+                 sampling_transforms=None, waveform_transforms=None, **kwargs):
         # store variable and static args
         self.variable_params = variable_params
         self.static_params = static_params
@@ -409,8 +409,9 @@ class BaseModel(metaclass=ABCMeta):
         params before being stored.
         """
         # add the static params
-        params.update(self.static_params)
-        self._current_params = self._transform_params(**params)
+        values = self.static_params.copy()
+        values.update(params)
+        self._current_params = self._transform_params(**values)
         self._current_stats = ModelStats()
 
     @property
