@@ -31,13 +31,15 @@ import numpy
 import itertools
 from scipy.interpolate import interp1d
 
-from pycbc.waveform import get_fd_waveform_sequence
+from pycbc.waveform import (get_fd_waveform_sequence,
+                            get_fd_det_waveform_sequence, fd_det_sequence)
 from pycbc.detector import Detector
 from pycbc.types import Array
 
 from .gaussian_noise import BaseGaussianNoise
 from .relbin_cpu import (likelihood_parts, likelihood_parts_v,
-                         likelihood_parts_multi, likelihood_parts_multi_v)
+                         likelihood_parts_multi, likelihood_parts_multi_v,
+                         likelihood_parts_det)
 from .tools import DistMarg
 
 
@@ -95,13 +97,25 @@ def setup_bins(f_full, f_lo, f_hi, chi=1.0, eps=0.5, gammas=None):
     # frequency grid points
     fbin = dphi2f(dphi_grid)
     # indices of frequency grid points in the FFT array
-    fbin_ind = numpy.unique(
-        [numpy.argmin(numpy.absolute(f_full - ff)) for ff in fbin]
-    )
+    fbin_ind = numpy.searchsorted(f_full, fbin)
+    for idx_fbin, idx_f_full in enumerate(fbin_ind):
+        if idx_f_full == 0:
+            curr_idx = 0
+        elif idx_f_full == len(f_full):
+            curr_idx = len(f_full) - 1
+        else:
+            abs1 = abs(f_full[idx_f_full] - fbin[idx_fbin])
+            abs2 = abs(f_full[idx_f_full-1] - fbin[idx_fbin])
+            if abs1 > abs2:
+                curr_idx = idx_f_full - 1
+            else:
+                curr_idx = idx_f_full
+        fbin_ind[idx_fbin] = curr_idx
+    fbin_ind = numpy.unique(fbin_ind)
     return fbin_ind
 
 
-class Relative(BaseGaussianNoise, DistMarg):
+class Relative(DistMarg, BaseGaussianNoise):
     r"""Model that assumes the likelihood in a region around the peak
     is slowly varying such that a linear approximation can be made, and
     likelihoods can be calculated at a coarser frequency resolution. For
@@ -160,7 +174,7 @@ class Relative(BaseGaussianNoise, DistMarg):
         **kwargs
     ):
 
-        variable_params, kwargs = self.setup_distance_marginalization(
+        variable_params, kwargs = self.setup_marginalization(
                                variable_params,
                                marginalize_phase=marginalize_phase,
                                **kwargs)
@@ -169,10 +183,17 @@ class Relative(BaseGaussianNoise, DistMarg):
             variable_params, data, low_frequency_cutoff, **kwargs
         )
 
+        # If the waveform handles the detector response internally, set
+        # self.det_response = True
+        self.no_det_response = False
+        if self.static_params['approximant'] in fd_det_sequence:
+            self.no_det_response = True
+
         # reference waveform and bin edges
         self.f, self.df, self.end_time, self.det = {}, {}, {}, {}
         self.h00, self.h00_sparse = {}, {}
         self.fedges, self.edges = {}, {}
+        self.ta = {}
         self.antenna_time = {}
 
         # filtered summary data for linear approximation
@@ -181,6 +202,7 @@ class Relative(BaseGaussianNoise, DistMarg):
         # store fiducial waveform params
         self.fid_params = self.static_params.copy()
         self.fid_params.update(fiducial_params)
+
         for k in self.static_params:
             if self.fid_params[k] == 'REPLACE':
                self.fid_params.pop(k)
@@ -191,7 +213,7 @@ class Relative(BaseGaussianNoise, DistMarg):
             self.f[ifo] = numpy.array(d0.sample_frequencies)
             self.df[ifo] = d0.delta_f
             self.end_time[ifo] = float(d0.end_time)
-            self.det[ifo] = Detector(ifo)
+            # self.det[ifo] = Detector(ifo)
 
             # generate fiducial waveform
             f_lo = self.kmin[ifo] * self.df[ifo]
@@ -204,14 +226,30 @@ class Relative(BaseGaussianNoise, DistMarg):
             # prune low frequency samples to avoid waveform errors
             fpoints = Array(self.f[ifo].astype(numpy.float64))
             fpoints = fpoints[self.kmin[ifo]:self.kmax[ifo]+1]
-            fid_hp, fid_hc = get_fd_waveform_sequence(sample_points=fpoints,
-                                                      **self.fid_params)
 
-            # check for zeros at high frequencies
+            if self.no_det_response:
+                wave = get_fd_det_waveform_sequence(ifos=ifo,
+                                                    sample_points=fpoints,
+                                                    **self.fid_params)
+                curr_wav = wave[ifo]
+            else:
+                fid_hp, fid_hc = get_fd_waveform_sequence(sample_points=fpoints,
+                                                          **self.fid_params)
+                curr_wav = fid_hp
+
+            # check for zeros at low and high frequencies
             # make sure only nonzero samples are included in bins
-            numzeros = list(fid_hp[::-1] != 0j).index(True)
-            if numzeros > 0:
-                new_kmax = self.kmax[ifo] - numzeros
+            numzeros_lo = list(curr_wav != 0j).index(True)
+            if numzeros_lo > 0:
+                new_kmin = self.kmin[ifo] + numzeros_lo
+                f_lo = new_kmin * self.df[ifo]
+                logging.info(
+                    "WARNING! Fiducial waveform starts above "
+                    "low-frequency-cutoff, initial bin frequency "
+                    "will be %s Hz", f_lo)
+            numzeros_hi = list(curr_wav[::-1] != 0j).index(True)
+            if numzeros_hi > 0:
+                new_kmax = self.kmax[ifo] - numzeros_hi
                 f_hi = new_kmax * self.df[ifo]
                 logging.info(
                     "WARNING! Fiducial waveform terminates below "
@@ -219,27 +257,36 @@ class Relative(BaseGaussianNoise, DistMarg):
                     "will be %s Hz", f_hi)
 
             # make copy of fiducial wfs, adding back in low frequencies
-            fid_hp.resize(len(self.f[ifo]))
-            fid_hc.resize(len(self.f[ifo]))
-            hp0 = numpy.roll(fid_hp, self.kmin[ifo])
-            hc0 = numpy.roll(fid_hc, self.kmin[ifo])
+            if self.no_det_response:
+                curr_wav.resize(len(self.f[ifo]))
+                curr_wav = numpy.roll(curr_wav, self.kmin[ifo])
+                # get detector-specific arrival times relative to end of data
+                self.ta[ifo] = -self.end_time[ifo]
+                tshift = numpy.exp(-2.0j * numpy.pi * self.f[ifo] * self.ta[ifo])
+                h00 = numpy.array(curr_wav) * tshift
+                self.h00[ifo] = h00
+            else:
+                fid_hp.resize(len(self.f[ifo]))
+                fid_hc.resize(len(self.f[ifo]))
+                hp0 = numpy.roll(fid_hp, self.kmin[ifo])
+                hc0 = numpy.roll(fid_hc, self.kmin[ifo])
 
-            # get detector-specific arrival times relative to end of data
-            dt = self.det[ifo].time_delay_from_earth_center(
-                self.fid_params["ra"],
-                self.fid_params["dec"],
-                self.fid_params["tc"],
-            )
+                self.det[ifo] = Detector(ifo)
+                dt = self.det[ifo].time_delay_from_earth_center(
+                    self.fid_params["ra"],
+                    self.fid_params["dec"],
+                    self.fid_params["tc"],
+                )
+                self.ta = self.fid_params["tc"] + dt - self.end_time[ifo]
 
-            ta = self.fid_params["tc"] + dt - self.end_time[ifo]
-            tshift = numpy.exp(-2.0j * numpy.pi * self.f[ifo] * ta)
+                fp, fc = self.det[ifo].antenna_pattern(
+                    self.fid_params["ra"], self.fid_params["dec"],
+                    self.fid_params["polarization"], self.fid_params["tc"])
 
-            fp, fc = self.det[ifo].antenna_pattern(
-            self.fid_params["ra"], self.fid_params["dec"],
-            self.fid_params["polarization"], self.fid_params["tc"])
+                tshift = numpy.exp(-2.0j * numpy.pi * self.f[ifo] * self.ta)
 
-            h00 = (hp0 * fp + hc0 * fc) * tshift
-            self.h00[ifo] = h00
+                h00 = (hp0 * fp + hc0 * fc) * tshift
+                self.h00[ifo] = h00
 
             # compute frequency bins
             logging.info("Computing frequency bins")
@@ -313,7 +360,10 @@ class Relative(BaseGaussianNoise, DistMarg):
             self.mlik = likelihood_parts_multi_v
         else:
             atimes = self.fid_params["tc"]
-            self.lik = likelihood_parts
+            if self.no_det_response:
+                self.lik = likelihood_parts_det
+            else:
+                self.lik = likelihood_parts
             self.mlik = likelihood_parts_multi
         return atimes
 
@@ -340,13 +390,21 @@ class Relative(BaseGaussianNoise, DistMarg):
     def get_waveforms(self, params):
         """ Get the waveform polarizations for each ifo
         """
+        if self.no_det_response:
+            wfs = {}
+            for ifo in self.data:
+                wfs.update(get_fd_det_waveform_sequence(
+                        ifos=ifo, sample_points=self.fedges[ifo], **params))
+            return wfs
+
         wfs = []
         for edge in self.edge_unique:
             hp, hc = get_fd_waveform_sequence(sample_points=edge, **params)
             hp = hp.numpy()
             hc = hc.numpy()
             wfs.append((hp, hc))
-        return {ifo: wfs[self.ifo_map[ifo]] for ifo in self.data}
+        wf_ret = {ifo: wfs[self.ifo_map[ifo]] for ifo in self.data}
+        return wf_ret
 
     @property
     def multi_signal_support(self):
@@ -355,7 +413,7 @@ class Relative(BaseGaussianNoise, DistMarg):
         """
         # Check if this model *can* be included in a multi-signal model.
         # All marginalizations must currently be disabled to work!
-        if (self.marginalize_vector or
+        if (self.marginalize_vector_params or
             self.marginalize_distance or
             self.marginalize_phase):
             logging.info("Cannot use single template model inside of"
@@ -435,34 +493,42 @@ class Relative(BaseGaussianNoise, DistMarg):
         p.update(self.static_params)
         wfs = self.get_waveforms(p)
 
-        hh = 0.0
-        hd = 0j
+        norm = 0.0
+        filt = 0j
         self._current_wf_parts = {}
         for ifo in self.data:
 
-            det = self.det[ifo]
             freqs = self.fedges[ifo]
             sdat = self.sdat[ifo]
             h00 = self.h00_sparse[ifo]
             end_time = self.end_time[ifo]
             times = self.antenna_time[ifo]
 
-            # project waveform to detector frame
-            hp, hc = wfs[ifo]
-            fp, fc = det.antenna_pattern(p["ra"], p["dec"],
-                                         p["polarization"], times)
-            dt = det.time_delay_from_earth_center(p["ra"], p["dec"], times)
-            dtc = p["tc"] + dt - end_time
+            # project waveform to detector frame if waveform does not deal
+            # with detector response. Otherwise, skip detector response.
 
-            hdp, hhp = self.lik(freqs, fp, fc, dtc,
-                                hp, hc, h00,
-                                sdat['a0'], sdat['a1'],
-                                sdat['b0'], sdat['b1'])
+            if self.no_det_response:
+                dtc = -end_time
+                channel = wfs[ifo].numpy()
+                filter_i, norm_i = self.lik(freqs, dtc, channel, h00,
+                                            sdat['a0'], sdat['a1'],
+                                            sdat['b0'], sdat['b1'])
+            else:
+                hp, hc = wfs[ifo]
+                det = self.det[ifo]
+                fp, fc = det.antenna_pattern(p["ra"], p["dec"],
+                                             p["polarization"], times)
+                dt = det.time_delay_from_earth_center(p["ra"], p["dec"], times)
+                dtc = p["tc"] + dt - end_time
 
-            self._current_wf_parts[ifo] = (fp, fc, dtc, hp, hc, h00)
-            hd += hdp
-            hh += hhp
-        return self.marginalize_loglr(hd, hh)
+                filter_i, norm_i = self.lik(freqs, fp, fc, dtc,
+                                            hp, hc, h00,
+                                            sdat['a0'], sdat['a1'],
+                                            sdat['b0'], sdat['b1'])
+                self._current_wf_parts[ifo] = (fp, fc, dtc, hp, hc, h00)
+            filt += filter_i
+            norm += norm_i
+        return self.marginalize_loglr(filt, norm)
 
     def write_metadata(self, fp, group=None):
         """Adds writing the fiducial parameters and epsilon to file's attrs.
