@@ -17,15 +17,19 @@
 This modules provides a list of implemented samplers for parameter estimation.
 """
 
+import logging
+
 # pylint: disable=unused-import
 from .base import (initial_dist_from_config, create_new_output_file)
 from .multinest import MultinestSampler
 from .ultranest import UltranestSampler
+from .dummy import DummySampler
 
 # list of available samplers
 samplers = {cls.name: cls for cls in (
     MultinestSampler,
     UltranestSampler,
+    DummySampler,
 )}
 
 try:
@@ -81,5 +85,9 @@ def load_from_config(cp, model, **kwargs):
     sampler :
         The initialized sampler.
     """
+    if len(model.variable_params) == 0:
+        logging.info('No variable params, so assuming Dummy Sampler')
+        return DummySampler.from_config(cp, model, **kwargs)
+
     name = cp.get('sampler', 'name')
     return samplers[name].from_config(cp, model, **kwargs)
