@@ -1,5 +1,3 @@
-import os.path
-
 from pycbc.tmpltbank.calc_moments import *
 from pycbc.tmpltbank.lambda_mapping import *
 from pycbc.tmpltbank.coord_utils import *
@@ -7,8 +5,4 @@ from pycbc.tmpltbank.lattice_utils import *
 from pycbc.tmpltbank.brute_force_methods import *
 from pycbc.tmpltbank.option_utils import *
 from pycbc.tmpltbank.partitioned_bank import *
-from pycbc.tmpltbank.em_progenitors import *
 from pycbc.tmpltbank.bank_conversions import *
-
-# Setup the directory with the NS equilibrium sequence(s)
-NS_SEQUENCE_FILE_DIRECTORY = os.path.join(os.path.dirname(__file__), 'ns_sequences')
