@@ -398,7 +398,7 @@ class FrequencySeries(Array):
                                         self.numpy().imag)).T
             _numpy.savetxt(path, output)
         elif ext == '.xml' or path.endswith('.xml.gz'):
-            from pycbc.io.live import make_psd_xmldoc
+            from pycbc.io.ligolw import make_psd_xmldoc
             from ligo.lw import utils
 
             if self.kind != 'real':
@@ -412,8 +412,11 @@ class FrequencySeries(Array):
             if not first_idx == 0:
                 data_lal[:first_idx] = data_lal[first_idx]
             psddict = {ifo: output}
-            utils.write_filename(make_psd_xmldoc(psddict), path,
-                                 gz=path.endswith(".gz"))
+            utils.write_filename(
+                make_psd_xmldoc(psddict),
+                path,
+                compress='auto'
+            )
         elif ext == '.hdf':
             key = 'data' if group is None else group
             with h5py.File(path, 'a') as f:
