@@ -251,8 +251,6 @@ class TemplateBank(object):
     has_compressed_waveforms : {False, bool}
         True if compressed waveforms are present in the the (hdf) file; False
         otherwise.
-    parameters : tuple
-        The parameters loaded from the input file. Same as `table.fieldnames`.
     indoc : {None, xmldoc}
         If an xml file was provided, an in-memory representation of the xml.
         Otherwise, None.
@@ -282,7 +280,7 @@ class TemplateBank(object):
             names = tuple([n if n!= 'alpha6' else 'f_lower' for n in names])
             self.table.dtype.names = names
 
-        elif ext.endswith(('hdf', '.h5')):
+        elif ext.endswith(('hdf', '.h5', '.hdf5')):
             self.indoc = None
             f = h5py.File(filename, 'r')
             self.filehandler = f
@@ -344,6 +342,9 @@ class TemplateBank(object):
 
     @property
     def parameters(self):
+        """tuple: The parameters loaded from the input file.
+        Same as `table.fieldnames`.
+        """
         return self.table.fieldnames
 
     def ensure_hash(self):
@@ -377,7 +378,8 @@ class TemplateBank(object):
         Parameters
         ----------
         filename : str
-            The name of the file to write to. Must end in '.hdf'.
+            The name of the file to write to. Must be a recognised HDF5
+            file extension
         start_index : If a specific slice of the template bank is to be
             written to the hdf file, this would specify the index of the
             first template in the slice
@@ -401,7 +403,7 @@ class TemplateBank(object):
         h5py.File
             The file handler to the output hdf file (left open).
         """
-        if not filename.endswith('.hdf'):
+        if not filename.endswith(('.hdf', '.h5', '.hdf5')):
             raise ValueError("Unrecoginized file extension")
         if os.path.exists(filename) and not force:
             raise IOError("File %s already exists" %(filename))
@@ -541,14 +543,14 @@ class LiveFilterBank(TemplateBank):
         Parameters
         ----------
         num : int
-            Proposed size of waveform in seconds
+            Proposed size of waveform in samples.
 
         Returns
         -------
         size: int
-            The rounded size to use for the waveform buffer in seconds. This
-        is calculaed using an internal `increment` attribute, which determines
-        the discreteness of the rounding.
+            The rounded size to use for the waveform buffer in samples.
+            This is calculated using an internal `increment` attribute, which
+            determines the discreteness of the rounding.
         """
         inc = self.increment
         size = np.ceil(num / self.sample_rate / inc) * self.sample_rate * inc
