@@ -3,7 +3,8 @@
 
 
 def add_custom_waveform(approximant, function, domain,
-                        sequence=False, force=False):
+                        sequence=False, has_det_response=False,
+                        force=False,):
     """ Make custom waveform available to pycbc
 
     Parameters
@@ -17,8 +18,11 @@ def add_custom_waveform(approximant, function, domain,
     sequence : bool, False
         Function evaluates waveform at only chosen points (instead of a
         equal-spaced grid).
+    has_det_response : bool, False
+        Check if waveform generator has built-in detector response.
     """
-    from pycbc.waveform.waveform import cpu_fd, cpu_td, fd_sequence
+    from pycbc.waveform.waveform import (cpu_fd, cpu_td, fd_sequence,
+                                         fd_det, fd_det_sequence)
 
     used = RuntimeError("Can't load plugin waveform {}, the name is"
                         " already in use.".format(approximant))
@@ -29,13 +33,23 @@ def add_custom_waveform(approximant, function, domain,
         cpu_td[approximant] = function
     elif domain == 'frequency':
         if sequence:
-            if not force and (approximant in fd_sequence):
-                raise used
-            fd_sequence[approximant] = function
+            if not has_det_response:
+                if not force and (approximant in fd_sequence):
+                    raise used
+                fd_sequence[approximant] = function
+            else:
+                if not force and (approximant in fd_det_sequence):
+                    raise used
+                fd_det_sequence[approximant] = function
         else:
-            if not force and (approximant in cpu_fd):
-                raise used
-            cpu_fd[approximant] = function
+            if not has_det_response:
+                if not force and (approximant in cpu_fd):
+                    raise used
+                cpu_fd[approximant] = function
+            else:
+                if not force and (approximant in fd_det):
+                    raise used
+                fd_det[approximant] = function
     else:
         raise ValueError("Invalid domain ({}), should be "
                          "'time' or 'frequency'".format(domain))
@@ -57,21 +71,34 @@ def add_length_estimator(approximant, function):
                            " already in use.".format(approximant))
     _filter_time_lengths[approximant] = function
 
+    from pycbc.waveform.waveform import td_fd_waveform_transform
+    td_fd_waveform_transform(approximant)
+
 
 def retrieve_waveform_plugins():
     """ Process external waveform plugins
     """
     import pkg_resources
 
-    # Check for fd waveforms
+    # Check for fd waveforms (no detector response)
     for plugin in pkg_resources.iter_entry_points('pycbc.waveform.fd'):
         add_custom_waveform(plugin.name, plugin.resolve(), 'frequency')
 
-    # Check for fd sequence waveforms
+    # Check for fd waveforms (has detector response)
+    for plugin in pkg_resources.iter_entry_points('pycbc.waveform.fd_det'):
+        add_custom_waveform(plugin.name, plugin.resolve(), 'frequency',
+                            has_det_response=True)
+
+    # Check for fd sequence waveforms (no detector response)
     for plugin in pkg_resources.iter_entry_points('pycbc.waveform.fd_sequence'):
         add_custom_waveform(plugin.name, plugin.resolve(), 'frequency',
                             sequence=True)
 
+    # Check for fd sequence waveforms (has detector response)
+    for plugin in pkg_resources.iter_entry_points('pycbc.waveform.fd_det_sequence'):
+        add_custom_waveform(plugin.name, plugin.resolve(), 'frequency',
+                            sequence=True, has_det_response=True)
+
     # Check for td waveforms
     for plugin in pkg_resources.iter_entry_points('pycbc.waveform.td'):
         add_custom_waveform(plugin.name, plugin.resolve(), 'time')
