@@ -36,10 +36,44 @@ import numpy as np
 from scipy.stats import rayleigh
 from ligo import segments
 from ligo.lw import ligolw, lsctables, utils
-from pycbc.workflow.core import File, FileList, resolve_url_to_file
+from gwdatafind.utils import filename_metadata
+from pycbc.workflow.core import \
+    File, FileList, resolve_url_to_file, Executable, Node
 from pycbc.workflow.jobsetup import select_generic_executable
 
 
+def select_grb_pp_class(wflow, curr_exe):
+    """
+    This function returns the class for PyGRB post-processing scripts.
+
+    Parameters
+    ----------
+    curr_exe : string
+        The name of the executable
+
+    Returns
+    -------
+    exe_class : Sub-class of pycbc.workflow.core.Executable that holds utility
+        functions appropriate for the given executable.  Instances of the class
+        ('jobs') **must** have methods
+        * job.create_node()
+        and
+        * job.get_valid_times(ifo, )
+    """
+    exe_path = wflow.cp.get('executables', curr_exe)
+    exe_name = os.path.basename(exe_path)
+    exe_to_class_map = {
+        'pycbc_grb_trig_combiner': PycbcGrbTrigCombinerExecutable,
+        'pycbc_grb_trig_cluster': PycbcGrbTrigClusterExecutable,
+        'pycbc_grb_inj_finder': PycbcGrbInjFinderExecutable,
+        'pycbc_grb_inj_combiner': PycbcGrbInjCombinerExecutable
+    }
+    if exe_name not in exe_to_class_map:
+        raise ValueError(f"No job class exists for executable {curr_exe}")
+
+    return exe_to_class_map[exe_name]
+
+
 def set_grb_start_end(cp, start, end):
     """
     Function to update analysis boundaries as workflow is generated
@@ -273,7 +307,7 @@ def make_gating_node(workflow, datafind_files, outdir=None, tags=None):
         input_files = FileList([datafind_file for datafind_file in \
                                 datafind_files if datafind_file.ifo == ifo])
         condition_strain_jobs = condition_strain_class(cp, "condition_strain",
-                ifo=ifo, out_dir=outdir, tags=tags)
+                ifos=ifo, out_dir=outdir, tags=tags)
         condition_strain_node, condition_strain_out = \
                 condition_strain_jobs.create_node(input_files, tags=tags)
         condition_strain_nodes.append(condition_strain_node)
@@ -365,3 +399,179 @@ def get_sky_grid_scale(
         if upscale:
             out *= rayleigh.interval(2 * containment - 1)[-1]
     return out
+
+
+def setup_pygrb_pp_workflow(wf, pp_dir, seg_dir, segment, insp_files,
+                            inj_files, inj_insp_files, inj_tags):
+    """
+    Generate post-processing section of PyGRB offline workflow
+    """
+    pp_outs = FileList([])
+    # Begin setting up trig combiner job(s)
+    # Select executable class and initialize
+    exe_class = select_grb_pp_class(wf, "trig_combiner")
+    job_instance = exe_class(wf.cp, "trig_combiner")
+    # Create node for coherent no injections jobs
+    node, trig_files = job_instance.create_node(wf.ifos, seg_dir, segment,
+                                    insp_files, pp_dir)
+    wf.add_node(node)
+    pp_outs.append(trig_files)
+
+    # Trig clustering for each trig file
+    exe_class = select_grb_pp_class(wf, "trig_cluster")
+    job_instance = exe_class(wf.cp, "trig_cluster")
+    for trig_file in trig_files:
+        # Create and add nodes
+        node, out_file = job_instance.create_node(trig_file, pp_dir)
+        wf.add_node(node)
+        pp_outs.append(out_file)
+
+    # Find injections from triggers
+    exe_class = select_grb_pp_class(wf, "inj_finder")
+    job_instance = exe_class(wf.cp, "inj_finder")
+    inj_find_files = FileList([])
+    for inj_tag in inj_tags:
+        tag_inj_files = FileList([f for f in inj_files
+                                  if inj_tag in f.tags])
+        # The here stems from the injection group information
+        # being stored in the second tag. This could be improved
+        # depending on the final implementation of injections
+        tag_insp_files = FileList([f for f in inj_insp_files
+                                   if inj_tag in f.tags[1]])
+        node, inj_find_file = job_instance.create_node(
+                                           tag_inj_files, tag_insp_files,
+                                           pp_dir)
+        wf.add_node(node)
+        inj_find_files.append(inj_find_file)
+    pp_outs.append(inj_find_files)
+
+    # Combine injections
+    exe_class = select_grb_pp_class(wf, "inj_combiner")
+    job_instance = exe_class(wf.cp, "inj_combiner")
+    inj_comb_files = FileList([])
+    for in_file in inj_find_files:
+        if 'DETECTION' not in in_file.tags:
+            node, inj_comb_file = job_instance.create_node(in_file,
+                                                           pp_dir,
+                                                           in_file.tags,
+                                                           segment)
+            wf.add_node(node)
+            inj_comb_files.append(inj_comb_file)
+    pp_outs.append(inj_comb_files)
+
+    return pp_outs
+
+
+class PycbcGrbTrigCombinerExecutable(Executable):
+    """ The class responsible for creating jobs
+    for ''pycbc_grb_trig_combiner''.
+    """
+
+    current_retention_level = Executable.ALL_TRIGGERS
+
+    def __init__(self, cp, name):
+        super().__init__(cp=cp, name=name)
+        self.trigger_name = cp.get('workflow', 'trigger-name')
+        self.trig_start_time = cp.get('workflow', 'start-time')
+        self.num_trials = int(cp.get('trig_combiner', 'num-trials'))
+
+    def create_node(self, ifo_tag, seg_dir, segment, insp_files,
+                    out_dir, tags=None):
+        node = Node(self)
+        node.add_opt('--verbose')
+        node.add_opt("--ifo-tag", ifo_tag)
+        node.add_opt("--grb-name", self.trigger_name)
+        node.add_opt("--trig-start-time", self.trig_start_time)
+        node.add_opt("--segment-dir", seg_dir)
+        node.add_input_list_opt("--input-files", insp_files)
+        node.add_opt("--user-tag", "PYGRB")
+        node.add_opt("--num-trials", self.num_trials)
+        # Prepare output file tag
+        user_tag = f"PYGRB_GRB{self.trigger_name}"
+        if tags:
+            user_tag += "_{}".format(tags)
+        # Add on/off source and off trial outputs
+        output_files = FileList([])
+        outfile_types = ['ALL_TIMES', 'OFFSOURCE', 'ONSOURCE']
+        for i in range(self.num_trials):
+            outfile_types.append("OFFTRIAL_{}".format(i+1))
+        for out_type in outfile_types:
+            out_name = "{}-{}_{}-{}-{}.h5".format(
+                       ifo_tag, user_tag, out_type,
+                       segment[0], segment[1]-segment[0])
+            out_file = File(ifo_tag, 'trig_combiner', segment,
+                            file_url=os.path.join(out_dir, out_name))
+            node.add_output(out_file)
+            output_files.append(out_file)
+
+        return node, output_files
+
+
+class PycbcGrbTrigClusterExecutable(Executable):
+    """ The class responsible for creating jobs
+    for ''pycbc_grb_trig_cluster''.
+    """
+
+    current_retention_level = Executable.ALL_TRIGGERS
+
+    def __init__(self, cp, name):
+        super().__init__(cp=cp, name=name)
+
+    def create_node(self, in_file, out_dir):
+        node = Node(self)
+        node.add_input_opt("--trig-file", in_file)
+        # Determine output file name
+        ifotag, filetag, segment = filename_metadata(in_file.name)
+        start, end = segment
+        out_name = "{}-{}_CLUSTERED-{}-{}.h5".format(ifotag, filetag,
+                                                     start, end-start)
+        out_file = File(ifotag, 'trig_cluster', segment,
+                        file_url=os.path.join(out_dir, out_name))
+        node.add_output(out_file)
+
+        return node, out_file
+
+
+class PycbcGrbInjFinderExecutable(Executable):
+    """The class responsible for creating jobs for ``pycbc_grb_inj_finder``
+    """
+    current_retention_level = Executable.ALL_TRIGGERS
+
+    def __init__(self, cp, exe_name):
+        super().__init__(cp=cp, name=exe_name)
+
+    def create_node(self, inj_files, inj_insp_files,
+                    out_dir, tags=None):
+        if tags is None:
+            tags = []
+        node = Node(self)
+        node.add_input_list_opt('--input-files', inj_insp_files)
+        node.add_input_list_opt('--inj-files', inj_files)
+        ifo_tag, desc, segment = filename_metadata(inj_files[0].name)
+        desc = '_'.join(desc.split('_')[:-1])
+        out_name = "{}-{}_FOUNDMISSED-{}-{}.h5".format(
+            ifo_tag, desc, segment[0], abs(segment))
+        out_file = File(ifo_tag, 'inj_finder', segment,
+                        os.path.join(out_dir, out_name), tags=tags)
+        node.add_output(out_file)
+        return node, out_file
+
+
+class PycbcGrbInjCombinerExecutable(Executable):
+    """The class responsible for creating jobs ``pycbc_grb_inj_combiner``
+    """
+    current_retention_level = Executable.ALL_TRIGGERS
+
+    def __init__(self, cp, exe_name):
+        super().__init__(cp=cp, name=exe_name)
+
+    def create_node(self, input_file, out_dir, ifo_tag, segment, tags=None):
+        if tags is None:
+            tags = []
+        node = Node(self)
+        node.add_input_opt('--input-files', input_file)
+        out_name = input_file.name.replace('.h5', '-FILTERED.h5')
+        out_file = File(ifo_tag, 'inj_combiner', segment,
+                        os.path.join(out_dir, out_name), tags=tags)
+        node.add_output_opt('--output-file', out_file)
+        return node, out_file
